package com.mcwl.web.controller.system;

import com.mcwl.common.core.domain.R;
import com.mcwl.resource.domain.vo.AdviceVo;
import com.mcwl.resource.domain.vo.AttentionAdviceVo;
import com.mcwl.resource.domain.vo.CommentAdviceVo;
import com.mcwl.resource.domain.vo.LikeAdviceVo;
import com.mcwl.resource.service.ISysAdviceService;
import io.swagger.annotations.ApiParam;
import lombok.RequiredArgsConstructor;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import javax.validation.constraints.NotNull;
import java.util.List;

/**
 * 消息通知
 */
@RestController
@RequestMapping("system/advice")
@RequiredArgsConstructor
public class SysAdviceController {

    private final ISysAdviceService sysAdviceService;

    /**
     * 根据当前用户获取新消息通知
     */
    @GetMapping("getUserNewMsg")
    public R<List<AdviceVo>> getUserNewMsg() {
        List<AdviceVo> adviceVo = sysAdviceService.getUserNewMsg();
        return R.ok(adviceVo);
    }

    /**
     * 根据当前用户获取系统公告
     */
    @GetMapping("getUserSystemNotice")
    public R<List<AdviceVo>> getUserSystemNotice() {
        List<AdviceVo> adviceVo = sysAdviceService.getUserSystemNotice();
        return R.ok(adviceVo);
    }

    /**
     * 根据当前用户获取所有通知
     */
    @GetMapping("getUserAllMsg")
    public R<List<AdviceVo>> getUserAllMsg() {
        List<AdviceVo> adviceVo = sysAdviceService.getUserAllMsg();
        return R.ok(adviceVo);
    }

    /**
     * 获取所有通知
     */
    @GetMapping("getAllMsg")
    public R<List<AdviceVo>> getAllMsg() {
        List<AdviceVo> adviceVo = sysAdviceService.getAllMsg();
        return R.ok(adviceVo);
    }


    /**
     * 获取评论通知
     */
    @GetMapping("getCommentMsg")
    public R<List<CommentAdviceVo>> getCommentMsg(@Valid
                                                  @NotNull(message = "类型不能为空")
                                                  @ApiParam(value = "类型 0模型 1工作流 2图片 3全部", required = true)
                                                  Integer productType) {
        List<CommentAdviceVo> adviceVo = sysAdviceService.getCommentMsg(productType);
        return R.ok(adviceVo);
    }

    /**
     * 获取点赞通知
     */
    @GetMapping("getLikeMsg")
    public R<List<LikeAdviceVo>> getLikeMsg(@Valid
                                            @NotNull(message = "类型不能为空")
                                            @ApiParam(value = "类型 0模型 1工作流 2图片 3评论 4全部", required = true)
                                            Integer productType) {
        List<LikeAdviceVo> likeAdviceVoList = sysAdviceService.getLikeMsg(productType);
        return R.ok(likeAdviceVoList);
    }


    /**
     * 获取关注通知
     */
    @GetMapping("getAttentionMsg")
    public R<List<AttentionAdviceVo>> getAttentionMsg() {
        List<AttentionAdviceVo> attentionAdviceVoList = sysAdviceService.getAttentionMsg();
        return R.ok(attentionAdviceVoList);
    }

}
