package com.mcwl.resource.service.impl;

import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.mcwl.common.core.domain.entity.SysUser;
import com.mcwl.common.utils.SecurityUtils;
import com.mcwl.resource.domain.ModelImage;
import com.mcwl.resource.domain.ModelProduct;
import com.mcwl.resource.domain.SysAdvice;
import com.mcwl.resource.domain.WorkFlow;
import com.mcwl.resource.domain.vo.AdviceVo;
import com.mcwl.resource.domain.vo.AttentionAdviceVo;
import com.mcwl.resource.domain.vo.CommentAdviceVo;
import com.mcwl.resource.domain.vo.LikeAdviceVo;
import com.mcwl.resource.mapper.SysAdviceMapper;
import com.mcwl.resource.service.ISysAdviceService;
import com.mcwl.resource.service.ModelImageService;
import com.mcwl.resource.service.ModelService;
import com.mcwl.resource.service.WorkFlowService;
import com.mcwl.system.domain.enums.AdviceEnum;
import com.mcwl.system.service.ISysUserService;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.ConcurrentHashMap;

/**
 * 消息管理 业务层处理
 */
@Service
@RequiredArgsConstructor
public class SysAdviceServiceImpl extends ServiceImpl<SysAdviceMapper, SysAdvice> implements ISysAdviceService {

    private final ISysUserService sysUserService;

    private final ModelService modelService;

    private final WorkFlowService workFlowService;

    private final ModelImageService modelImageService;

    private final Map<Long, ModelProduct> modelProductMap = new ConcurrentHashMap<>();
    private final Map<Long, WorkFlow> workFlowMap = new ConcurrentHashMap<>();
    private final Map<Long, ModelImage> modelImageMap = new ConcurrentHashMap<>();


    @Override
    public List<AdviceVo> getUserNewMsg() {

        Long userId = SecurityUtils.getUserId();

        List<SysAdvice> sysAdvices = baseMapper.selectList(lambdaQuery()
                .eq(SysAdvice::getReceiverId, userId)
                .eq(SysAdvice::getType, AdviceEnum.NEW_MESSAGE_REMIND));

        return BeanUtil.copyToList(sysAdvices, AdviceVo.class);
    }

    @Override
    public List<AdviceVo> getUserSystemNotice() {
        Long userId = SecurityUtils.getUserId();

        List<SysAdvice> sysAdvices = baseMapper.selectList(lambdaQuery()
                .eq(SysAdvice::getReceiverId, userId)
                .eq(SysAdvice::getType, AdviceEnum.SYSTEM_NOTICE));

        return BeanUtil.copyToList(sysAdvices, AdviceVo.class);
    }

    @Override
    public List<AdviceVo> getUserAllMsg() {
        Long userId = SecurityUtils.getUserId();

        List<SysAdvice> sysAdvices = baseMapper.selectList(lambdaQuery()
                .eq(SysAdvice::getReceiverId, userId));

        return BeanUtil.copyToList(sysAdvices, AdviceVo.class);
    }

    @Override
    public List<AdviceVo> getAllMsg() {

        List<SysAdvice> sysAdvices = baseMapper.selectList(null);

        return BeanUtil.copyToList(sysAdvices, AdviceVo.class);
    }

    @Override
    public List<CommentAdviceVo> getCommentMsg(Integer productType) {

        this.postConstruct();

        List<SysAdvice> sysAdviceList = baseMapper.selectList(new LambdaQueryWrapper<SysAdvice>()
                .eq(SysAdvice::getType, AdviceEnum.COMMENT_REMIND)
                .eq(SysAdvice::getReceiverId, SecurityUtils.getUserId())
                .eq(productType != 3, SysAdvice::getProductType, productType));


        List<Long> commentUserIdList = new ArrayList<>();
        for (SysAdvice sysAdvice : sysAdviceList) {
            commentUserIdList.add(sysAdvice.getSenderId());
        }
        List<SysUser> sysUsers = sysUserService.selectUserByIds(commentUserIdList);

        Map<Long, SysUser> commentUserMap = new HashMap<>();
        for (SysUser sysUser : sysUsers) {
            commentUserMap.put(sysUser.getUserId(), sysUser);
        }

        List<CommentAdviceVo> commentAdviceVoList = new ArrayList<>();
        for (SysAdvice sysAdvice : sysAdviceList) {
            CommentAdviceVo commentAdviceVo = BeanUtil.copyProperties(sysAdvice, CommentAdviceVo.class);

            Long senderId = sysAdvice.getSenderId();
            SysUser senderUser = commentUserMap.get(senderId);
            commentAdviceVo.setUserAvatar(senderUser.getAvatar());
            if (productType == 0) {
                ModelProduct modelProduct = modelProductMap.get(sysAdvice.getProductId());
                if (Objects.nonNull(modelProduct)) {
                    commentAdviceVo.setProductImag(modelProduct.getSurfaceUrl());
                }
            } else if (productType == 1) {
                WorkFlow workFlow = workFlowMap.get(sysAdvice.getProductId());
                if (Objects.nonNull(workFlow)) {
                    commentAdviceVo.setProductImag(workFlow.getCoverPath());
                }
            } else if (productType == 2) {
                ModelImage modelImage = modelImageMap.get(sysAdvice.getProductId());
                if (Objects.nonNull(modelImage)) {
                    commentAdviceVo.setProductImag(modelImage.getImagePaths().split(",")[0]);
                }
            }

            commentAdviceVoList.add(commentAdviceVo);
        }

        return commentAdviceVoList;
    }

    @Override
    public List<LikeAdviceVo> getLikeMsg(Integer productType) {
        this.postConstruct();
        List<SysAdvice> sysAdviceList;
        if (productType == 3) {
            sysAdviceList = baseMapper.selectList(new LambdaQueryWrapper<SysAdvice>()
                    .eq(SysAdvice::getType, AdviceEnum.LIKE_REMIND)
                    .eq(SysAdvice::getReceiverId, SecurityUtils.getUserId())
                    .isNotNull(SysAdvice::getCommentId));
        } else {
            sysAdviceList = baseMapper.selectList(new LambdaQueryWrapper<SysAdvice>()
                    .eq(SysAdvice::getType, AdviceEnum.LIKE_REMIND)
                    .eq(SysAdvice::getReceiverId, SecurityUtils.getUserId())
                    .eq(productType != 4, SysAdvice::getProductType, productType));
        }

        List<Long> likeUserIdList = new ArrayList<>();
        for (SysAdvice sysAdvice : sysAdviceList) {
            likeUserIdList.add(sysAdvice.getSenderId());
        }
        List<SysUser> sysUsers = sysUserService.selectUserByIds(likeUserIdList);

        Map<Long, SysUser> likeUserMap = new HashMap<>();
        for (SysUser sysUser : sysUsers) {
            likeUserMap.put(sysUser.getUserId(), sysUser);
        }


        List<LikeAdviceVo> likeAdviceVoList = new ArrayList<>();
        for (SysAdvice sysAdvice : sysAdviceList) {
            LikeAdviceVo likeAdviceVo = BeanUtil.copyProperties(sysAdvice, LikeAdviceVo.class);

            Long senderId = sysAdvice.getSenderId();
            SysUser senderUser = likeUserMap.get(senderId);
            likeAdviceVo.setUserAvatar(senderUser.getAvatar());
            if (productType == 0) {
                ModelProduct modelProduct = modelProductMap.get(sysAdvice.getProductId());
                if (Objects.nonNull(modelProduct)) {
                    likeAdviceVo.setProductImag(modelProduct.getSurfaceUrl());
                }
            } else if (productType == 1) {
                WorkFlow workFlow = workFlowMap.get(sysAdvice.getProductId());
                if (Objects.nonNull(workFlow)) {
                    likeAdviceVo.setProductImag(workFlow.getCoverPath());
                }
            } else if (productType == 2) {
                ModelImage modelImage = modelImageMap.get(sysAdvice.getProductId());
                if (Objects.nonNull(modelImage)) {
                    likeAdviceVo.setProductImag(modelImage.getImagePaths().split(",")[0]);
                }
            }

            likeAdviceVoList.add(likeAdviceVo);
        }

        return likeAdviceVoList;
    }

    @Override
    public List<AttentionAdviceVo> getAttentionMsg() {
        List<SysAdvice> sysAdviceList = baseMapper.selectList(new LambdaQueryWrapper<SysAdvice>()
                .eq(SysAdvice::getType, AdviceEnum.FANS_REMIND)
                .eq(SysAdvice::getReceiverId, SecurityUtils.getUserId()));

        List<AttentionAdviceVo> attentionAdviceVoList = new ArrayList<>();
        List<Long> atentUserIdList = new ArrayList<>();

        // 通知表中获取所有关注我的用户id
        for (SysAdvice sysAdvice : sysAdviceList) {
            atentUserIdList.add(sysAdvice.getSenderId());
        }

        // 构建map, key为用户id，value为用户信息
        Map<Long, SysUser> attentionUserMap = new HashMap<>();

        // 批量查询用户信息, 并构建映射关系
        List<SysUser> atentUserList = sysUserService.listByIds(atentUserIdList);
        for (SysUser sysUser : atentUserList) {
            attentionUserMap.put(sysUser.getUserId(), sysUser);
        }

        for (SysAdvice sysAdvice : sysAdviceList) {
            AttentionAdviceVo attentionAdviceVo = BeanUtil.copyProperties(sysAdvice, AttentionAdviceVo.class);
            Long senderId = sysAdvice.getSenderId();
            SysUser attentionUser = attentionUserMap.get(senderId);
            attentionAdviceVo.setUserAvatar(attentionUser.getAvatar());
            attentionAdviceVoList.add(attentionAdviceVo);
        }

        return attentionAdviceVoList;
    }

    public void postConstruct() {
        Long userId = SecurityUtils.getUserId();

        // 并行加载数据
        CompletableFuture<List<ModelProduct>> productsFuture = CompletableFuture.supplyAsync(() ->
                modelService.lambdaQuery().eq(ModelProduct::getUserId, userId).list()
        );

        CompletableFuture<List<WorkFlow>> workflowsFuture = CompletableFuture.supplyAsync(() ->
                workFlowService.lambdaQuery().eq(WorkFlow::getUserId, userId).list()
        );

        CompletableFuture<List<ModelImage>> imagesFuture = CompletableFuture.supplyAsync(() ->
                modelImageService.lambdaQuery().eq(ModelImage::getUserId, userId).list()
        );

        // 等待所有任务完成
        CompletableFuture.allOf(productsFuture, workflowsFuture, imagesFuture).join();

        // 更新缓存
        modelProductMap.clear();
        productsFuture.join().forEach(p -> modelProductMap.put(p.getId(), p));

        workFlowMap.clear();
        workflowsFuture.join().forEach(w -> workFlowMap.put(w.getId(), w));

        modelImageMap.clear();
        imagesFuture.join().forEach(i -> modelImageMap.put(i.getId(), i));

    }


}
